// src/index.ts
import { AsyncAdapter as AsyncAdapter2, AsyncProvider } from "@stenodb/node";
import fp from "fastify-plugin";

// src/plugin.ts
import { AsyncAdapter, NodeProvider } from "@stenodb/node";
import { targetConstructorToSchema } from "class-validator-jsonschema";
var StenoPlugin = class {
  #fastify;
  #options;
  #db;
  #providers;
  constructor(fastify, options) {
    this.#fastify = fastify;
    this.#options = options;
    this.#db = new NodeProvider(options);
    this.#providers = /* @__PURE__ */ new Map();
    this.#fastify.decorate("steno", { get: this.getDatabase.bind(this) });
  }
  static async createInstance(fastify, options, done) {
    const db = new StenoPlugin(fastify, options);
    await db.registerDatabases();
    done();
    return db;
  }
  async registerDatabases() {
    this.registerEntities();
    for (const adapter of this.#options.adapters) {
      let provider;
      if (adapter instanceof AsyncAdapter) {
        provider = await this.#db.create(adapter);
        await provider.read();
      } else {
        throw new TypeError("Invalid adapter");
      }
      this.addSchema(adapter.entity);
      this.#providers.set(adapter.fileName, provider);
    }
  }
  registerEntities() {
    if (!this.#options.entities)
      return;
    for (const entity of this.#options.entities) {
      this.addSchema(entity);
    }
  }
  addSchema(entity) {
    if (this.#fastify.getSchema(entity.name))
      return;
    const schema = targetConstructorToSchema(
      entity,
      this.#options.entityOptions
    );
    if (!schema.properties)
      return;
    this.#fastify.addSchema({ ...schema, $id: entity.name });
  }
  getDatabase(name) {
    return this.#providers.get(name);
  }
};

// src/index.ts
var FastifySteno = fp(StenoPlugin.createInstance, {
  name: "@stenodb/fastify",
  fastify: "4.x"
});
var src_default = StenoPlugin.createInstance;
export {
  AsyncAdapter2 as AsyncAdapter,
  AsyncProvider,
  FastifySteno,
  src_default as default
};
